==> vilistextum-2.6.0.tar.bz2: vilistextum-2.6.0/src/charset.c <==
/*
 * Copyright (C) 1998-2002 Patric Mller
 * bhaak@gmx.net
 * http://www.diax.ch/users/bhaak/vilistextum/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 *
 *  history
 *  14.10.2001 : creation of this file
 */

#include <../config.h>

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <locale.h>
#include <stdlib.h>

#include <iconv.h>

#include "main.h"
#include "text.h"
#include "multibyte.h"

char *default_charset = "iso-8859-1";
char *default_country = "en_US.";
char *default_locale = "en_US.iso-8859-1";

char this_locale[DEF_STR_LEN],
	iconv_charset[DEF_STR_LEN],
	this_complete_locale[DEF_STR_LEN];

int usr=0;

iconv_t conv;

// ------------------------------------------------
#ifdef MULTIBYTE
void convert_character(int num, CHAR *outstring)
{
	char in[33], out[33];
	size_t result=(size_t)(-1);
	int i;
#ifdef HAVE_LIBICONV
	char *inp, *outp;
	size_t insize = 1, outsize = 32;
	char *ret;

#ifdef proc_debug
	printf("convert_character start\n");
#endif

	//#define iconv_debug 1
	
	ret = setlocale(LC_CTYPE, "en_US.utf-8"); 
  if (ret==NULL) { fprintf(stderr, "setlocale failed with: en_US.utf8\n\n"); }
  else { /*fprintf(stderr, "%s\n", ret);*/ } 

	for (i=0; i<33; i++) { in[i]=0x00; out[i]=0x00; }

	inp  = in;
	outp = out;
	insize = wctomb(inp, num);
#ifdef iconv_debug
	printf("=================================================\n1 insize %d %d\n", insize, num);
	printf("iconv_charset %s\n", iconv_charset);
#endif
	if ((conv = iconv_open(iconv_charset, "utf-8"))==(iconv_t)(-1)) 
 	{	perror("iconv_open failed"); } 
#ifdef iconv_debug
	printf("2\n");

	printf("iconv 1 %d %d \n", inp[0], inp[1]);
#endif
	result = iconv(conv, &inp, &insize, &outp, &outsize);
#ifdef iconv_debug
	printf("iconv 2\n");
#endif
#else
#ifdef iconv_debug
	printf("no iconv\n");
#endif
	for (i=0; i<33; i++) { in[i]=0x00; out[i]=0x00; }
#endif
	if (result==(size_t)(-1)) 
	{	
#ifdef iconv_debug
		printf("&#%d;", num);
#endif
		swprintf (outstring, 32, L"&#%d;\0", num);
		fprintf(stdout, "%ls\n", outstring);
	} else {
#ifdef iconv_debug
		printf("Input '%s'; Output '%s'\n", in, out);
#endif
		setlocale(LC_CTYPE, this_complete_locale);
		result = mbstowcs(outstring, out, strlen(out));
		outstring[result] = L'\0';
#ifdef iconv_debug
		printf("Result %d; outstring: %ls: out %s\nstrlen(out) %d; this_locale: %s\n", result, outstring, out, strlen(out), this_locale);
#endif		
	}
#ifdef iconv_debug
	printf("fertig %ls\n", outstring);
#endif
	iconv_close(conv);

#ifdef proc_debug
	printf("convert_character end\n\n");
#endif
} // end convert_character
#endif
// ------------------------------------------------

void set_locale(char *charset)
{
  // if charset == "", take the value from the environment variables
  char *ret;
  //ret = setlocale(LC_CTYPE, "");
	char country_charset[DEF_STR_LEN];
	if (usr==0)
	{
#ifdef proc_debug
	printf("set_locale start\n");
#endif

	// set charset for iconv conversion
	strcpy(iconv_charset, charset);
	if (transliteration) { strcat(iconv_charset, "//TRANSLIT");}
	//printf("iconv_charset %s\n", iconv_charset);

	strcpy(country_charset, default_country);
	strcat(country_charset, charset);

  ret = setlocale(LC_CTYPE, country_charset); 
	//printf("Locale: =%s=\n", country_charset);
	strcpy(this_complete_locale, country_charset);
	strcpy(this_locale, charset);
  if (ret==NULL) { fprintf(stderr, "setlocale failed with: %s\n\n", country_charset); }
  else { /*fprintf(stderr, "%s\n", ret);*/ } 
#ifdef proc_debug
	printf("set_locale end\n");
#endif
	}
} // end set_locale

// ------------------------------------------------

void use_locale()  { /*printf("using this locale %s\n", this_locale);*/ set_locale(this_locale); }
void use_default_locale() { set_locale(default_charset); }

// ------------------------------------------------

void set_usr_locale(char *user_locale) 
{
  set_locale(user_locale);
	usr=1;
}

// ------------------------------------------------
#ifdef MULTIBYTE
void strip_wchar(CHAR *locale, char *stripped_locale)
{
	CHAR *in  = locale;
	char *out = stripped_locale;
	int len = STRLEN(locale);
	int i;
#ifdef proc_debug
	printf("strip_wchar start\n");
#endif
	// copy stripped string to out
	for (i=0; i<len; i++) { out[i] = wctob(in[i]); }
	out[i] = 0x00;
	//printf("out %s\n", stripped_locale);
#ifdef proc_debug
	printf("strip_wchar end\n");
#endif
} // end strip_wchar
#endif
// ------------------------------------------------

void print_locale() { 
	//printf("%s\n", this_locale);
}

// ------------------------------------------------

==> vilistextum-2.6.2.tar.bz2: vilistextum-2.6.2/src/charset.c <==
/*
 * Copyright (C) 1998-2002 Patric Mller
 * bhaak@gmx.net
 * http://www.mysunrise.ch/users/bhaak/vilistextum/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 *
 *  history
 *  14.10.2001 : creation of this file
 */

#include <../config.h>

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <locale.h>
#include <stdlib.h>

#include <iconv.h>

#include "main.h"
#include "text.h"
#include "multibyte.h"

char *default_charset = "iso-8859-1";
char *default_country = "en_US.";
char *default_locale = "en_US.iso-8859-1";

char this_locale[DEF_STR_LEN],
	iconv_charset[DEF_STR_LEN],
	this_complete_locale[DEF_STR_LEN];

int usr=0;

iconv_t conv;

// ------------------------------------------------
#ifdef MULTIBYTE
int convert_character(int num, CHAR *outstring)
{
	char in[33], out[33];
	size_t result=(size_t)(-1);
	int i;
	int converted; // has the entity been successfully converted
#ifdef HAVE_LIBICONV
	char *inp, *outp;
	size_t insize = 1, outsize = 32;
	char *ret;

#ifdef proc_debug
	printf("convert_character start\n");
#endif

	//#define iconv_debug 1
	
	ret = setlocale(LC_CTYPE, "en_US.utf-8"); 
  if (ret==NULL) { fprintf(stderr, "setlocale failed with: en_US.utf8\n\n"); }
  else { /*fprintf(stderr, "%s\n", ret);*/ } 

	for (i=0; i<33; i++) { in[i]=0x00; out[i]=0x00; }
	//#define iconv_debug 1
	inp  = in;
	outp = out;
	insize = wctomb(inp, num);

	if ((conv = iconv_open(iconv_charset, "utf-8"))==(iconv_t)(-1)) 
 	{	printf("iconv_open failed: wrong character set?\n"); perror(iconv_charset); exit(1); } 

	result = iconv(conv, &inp, &insize, &outp, &outsize);
#else
	for (i=0; i<33; i++) { in[i]=0x00; out[i]=0x00; }
#endif
	if (result==(size_t)(-1)) 
	{	
#ifdef iconv_debug
		printf("not converted: &#%d;", num);
#endif
		//swprintf (outstring, 32, L"&#%d;\0", num);
		//fprintf(stdout, "%ls\n", outstring);
		converted = 0;
	} else {
#ifdef iconv_debug
		printf("Input '%s'; Output '%s'\n", in, out);
#endif
		converted = 1;
		setlocale(LC_CTYPE, this_complete_locale);
		result = mbstowcs(outstring, out, strlen(out));
		outstring[result] = L'\0';
#ifdef iconv_debug
		printf("Result %d; outstring: %ls: out %s\nstrlen(out) %d; this_locale: %s\n", result, outstring, out, strlen(out), this_locale);
#endif		
	}
#ifdef iconv_debug
	printf("fertig %d -%ls-\n", converted, outstring);
#endif
	iconv_close(conv);

#ifdef proc_debug
	printf("convert_character end\n\n");
#endif
	return(converted);
} // end convert_character
#endif
// ------------------------------------------------

void set_locale(char *charset)
{
  // if charset == "", take the value from the environment variables
  char *ret;
  //ret = setlocale(LC_CTYPE, "");
	char country_charset[DEF_STR_LEN];
	if (usr==0)
	{
#ifdef proc_debug
	printf("set_locale start\n");
#endif

	// set charset for iconv conversion
	strcpy(iconv_charset, charset);
	if (transliteration) { strcat(iconv_charset, "//TRANSLIT");}
	//printf("iconv_charset %s\n", iconv_charset);

	strcpy(country_charset, default_country);
	strcat(country_charset, charset);

  ret = setlocale(LC_CTYPE, country_charset); 
	//printf("Locale: =%s=\n", country_charset);
	strcpy(this_complete_locale, country_charset);
	strcpy(this_locale, charset);
  if (ret==NULL) { fprintf(stderr, "setlocale failed with: %s\n\n", country_charset); }
  else { /*fprintf(stderr, "%s\n", ret);*/ } 
#ifdef proc_debug
	printf("set_locale end\n");
#endif
	}
} // end set_locale

// ------------------------------------------------

void use_locale()  { /*printf("using this locale %s\n", this_locale);*/ set_locale(this_locale); }
void use_default_locale() { set_locale(default_charset); }

// ------------------------------------------------

void set_usr_locale(char *user_locale) 
{
  set_locale(user_locale);
	usr=1;
}

// ------------------------------------------------
#ifdef MULTIBYTE
void strip_wchar(CHAR *locale, char *stripped_locale)
{
	CHAR *in  = locale;
	char *out = stripped_locale;
	int len;
	int i;
#ifdef proc_debug
	printf("strip_wchar start\n");
#endif

	//printf("locale: -%d-%d-\n", locale[0], locale[1]);

	len = STRLEN(locale);
	// copy stripped string to out
	for (i=0; i<len; i++) { out[i] = wctob(in[i]); }
	out[i] = 0x00;
	//printf("out %s\n", stripped_locale);
#ifdef proc_debug
	printf("strip_wchar end\n");
#endif
} // end strip_wchar
#endif
// ------------------------------------------------

void print_locale() { 
	//printf("%s\n", this_locale);
}

// ------------------------------------------------
